/*
   Dot version 1.0  Copyright (C) 1999 Frank Reglin

   Dot is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   Dot is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
   See the GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with Dot; see the file COPYING.
*/
/* dotos2 is a deskpic module. OS/2 16 bit, msc6 */

/***************************************************************************\
* DOT.C - A Desktop Screen Saver extension by Frank Reglin (and John Ridges)
*         idea by Bettina Reglin
\***************************************************************************/


#define INCL_BASE
#define INCL_PM

#include <os2.h>
#include <stdlib.h>

typedef struct 
{
   HAB deskpichab;
   HWND screenhwnd;
   RECTL screenrectl;
   ULONG volatile closesemaphore;
   HMODULE thismodule;
} SAVERBLOCK;

char far * far pascal _loadds saverstatus(SAVERBLOCK far *, BOOL far *);
MRESULT CALLBACK saverdialog(HWND, USHORT, MPARAM, MPARAM);
void far pascal _loadds saverthread(void);

static SAVERBLOCK far *saverinfo;

/*
   This is the record that contains the options of the screen saver.
   These options are kept in the OS2.INI profile file.
*/
typedef struct 
{
   BOOL enabled;   /* The screen saver enabled status */
} PROFILEREC;

static char name[] = "Dot";  /* The name of this screen saver */
static BOOL gotprofile = FALSE;  /* Indicates that we've read the profile */
static PROFILEREC profile = { TRUE };   /* Default values */

static long xpos = -1;   /* dot position */
static long ypos = -1;   /* dot position */

#define DOTSIZE 4        /* DOT = DOTSIZE*DOTSIZE pixels */



char far * far pascal _loadds saverstatus(initptr,enabledptr) /*FOLD00*/
SAVERBLOCK far *initptr;
BOOL far *enabledptr;
{
SHORT i;
   
   /* Save the SAVERBLOCK address locally */
   saverinfo = initptr;

   /* Read the profile (but only once!) */
   if (!gotprofile) 
   {
      i = sizeof(PROFILEREC);
      WinQueryProfileData(saverinfo->deskpichab,"Deskpic",name,&profile,&i);
      gotprofile = TRUE;
   }
   
   /* Return the enabled status */
   *enabledptr = profile.enabled;
   
   /* Return the screen saver name */
   return name;
}

MRESULT CALLBACK saverdialog(hwnd,message,mp1,mp2) /*FOLD00*/
HWND hwnd;
USHORT message;
MPARAM mp1,mp2;
{
RECTL ourrect,desktoprect;

   switch(message) 
   {
   case WM_INITDLG:
      /* Center the dialog inside of the desktop */
      WinQueryWindowRect(hwnd,&ourrect);
      WinQueryWindowRect(HWND_DESKTOP,&desktoprect);
      WinSetWindowPos(hwnd,NULL,
         (int)(desktoprect.xRight-ourrect.xRight+ourrect.xLeft)/2,
         (int)(desktoprect.yTop-ourrect.yTop+ourrect.yBottom)/2,
         0,0,SWP_MOVE);

      /* Check the enabled button if enabled */
      if (profile.enabled)
         WinSendDlgItemMsg(hwnd,3,BM_SETCHECK,MPFROMSHORT(1),0);

      /* Bring up the dialog */
      WinShowWindow(hwnd,TRUE);
      return FALSE;

   case WM_COMMAND:
      /* If OK is pushed */
      if (SHORT1FROMMP(mp1) == 1) {

         /* Get the enabled status */
         profile.enabled =
            SHORT1FROMMR(WinSendDlgItemMsg(hwnd,3,BM_QUERYCHECK,0,0));

         /* Write the profile data */
         WinWriteProfileData(saverinfo->deskpichab,"Deskpic",name,&profile,
            sizeof(PROFILEREC));
      }
   }
   return WinDefDlgProc(hwnd,message,mp1,mp2);
}



void setdot( HPS hps, long x, long y, long color ) /*FOLD00*/
{
long x0, y0;
int xx, yy;
POINTL aptl;


   GpiSetColor( hps, color );

   /* origin, should be 0,0 : */
   x0 = saverinfo->screenrectl.xLeft;
   y0 = saverinfo->screenrectl.yBottom;

   for( xx = 0; xx < DOTSIZE; xx++ )
   {
      aptl.x = x0 + x * DOTSIZE + xx;
      for( yy = 0; yy < DOTSIZE; yy++ )
      {
         aptl.y = y0 + y * DOTSIZE + yy;
         GpiSetPel( hps, &aptl );
      }
   }
}


void far pascal _loadds saverthread() /*FOLD00*/
{
HAB hab;
HPS hps;
long x0, y0, xmax,ymax;

   /* Get an HAB for this thread (since we make PM calls) */
   hab = WinInitialize(0);

   /* Get an HPS of the screen */
   hps = WinGetPS(saverinfo->screenhwnd);

   /* Paint the screen black */
   WinFillRect(hps,&saverinfo->screenrectl,CLR_BLACK);

   /* origin, should be 0,0 : */
   x0 = saverinfo->screenrectl.xLeft;
   y0 = saverinfo->screenrectl.yBottom;

   /* max coordinates, relative to the origin: */
   xmax = (saverinfo->screenrectl.xRight - x0) / DOTSIZE - 1;
   ymax = (saverinfo->screenrectl.yTop - y0) / DOTSIZE;
   if( !(ymax & 1L) )
      ymax--;                       /* has to be odd ! */
   
   /* If the dot's position is unknown, initialize it */
   if( xpos < 0 )
   {
      xpos = 0;
      ypos = ymax;
   }

   /* Loop until DESKPIC tells us to stop */
   while (!saverinfo->closesemaphore)
   {
      /* Clear the dot at the old position: */
      setdot( hps, xpos, ypos, CLR_BLACK );

      /* Find the new position of the dot */
      if( ypos & 1L )
      {              /* odd: */
         xpos++;
         if( xpos > xmax )
         {
            xpos = xmax;
            ypos--;
         }
      }
      else
      {              /* even: */
         xpos--;
         if( xpos < 0 )
         {
            xpos = 0;
            ypos--;
            if( ypos < 0 )
               ypos = ymax;
        }
      }

      /* Draw the dot at the new position: */
      setdot( hps, xpos, ypos, CLR_WHITE );
   
      DosSleep( 32 );
   }

   /* Release the HPS of the screen */
   WinReleasePS(hps);

   /* Get rid of the HAB */
   WinTerminate(hab);

   /* Make sure the stack doesn't vanish before we're completely gone */
   DosEnterCritSec();

   /* Tell DESKPIC that we're gone */
   DosSemClear((HSEM)&saverinfo->closesemaphore);
}

